/*---------------------------------------------------------------------------
 *    This is the NDAS writer program. It is a DMT monitor. (by B. Mours)
----------------------------------------------------------------------------*/
#include <stdio.h>
#include <time.h>
#include <stdarg.h>
#include <dirent.h>
#include <unistd.h>
#include "FrameL.h"
#define NWMAXMSG 100

char *NwVersion = "v2.0 Aug 13, 02";

struct NwParam{
  int framePerFile;       /* number of frame per file                   */
  int freqLSC;            /* frequency for the LSC channel in kHz       */
  int htmlPeriod;         /* update period for the html status report   */
  int nFrame;             /* number of procesed frames                  */
  FrFile *file;           /* pointer to the frameFile                   */
  char frFName[256];      /* frame file name                            */
  char *htmlName;         /* full name for the html file                */
  char *dataDir;          /* directory for the data files               */ 
  char *lscName;          /* name of the lsc channel to be used         */
  int nLogMsg;            /* number of log messages                     */
  char *tag;              /* tag used to define the channels written    */
  char *logMsg[NWMAXMSG]; /* hold log messages                          */
  char config[2048];      /* hold configuration used                    */
  int lifeTime;           /* give time for which the file are keept     */
  int    nCall;           /* number of calls                            */
  int    nAdcF;           /* number of fast Adc                         */
  int    nAdcS;           /* number of slow Adc                         */
  int startTime;          /* starting time of the current file          */
  FRBOOL verbose;         /* if no supress the printf                   */
};

struct NwParam NwP;

/**--------------------------------------------------------------------------*/
void DMT_Initialize(int argc, const char *argv[]) {
/**--------------------------------------------------------------------------*/
/* options are:                                                              */
/*   -n number of frames par file (default: 1000)                            */
/*   -f frequency (in kHz for the main channel)                              */
/*   -s scaling factor for the main channel for the float ->int conversion   */
/*   -v verbose to produse some terminal ouput                               */
/*   -d directory for the data frame files                                  */
/*   -p  update period for the html report (default 5 s)                     */
/**--------------------------------------------------------------------------*/
 int i;

           /*-------------------- Default values ----------------------------*/

 NwP.framePerFile  = 1000;
 NwP.freqLSC    = 2;
 NwP.nFrame     = 0;
 NwP.verbose    = FR_NO;
 NwP.lscName    = NULL;
 NwP.htmlPeriod = 10;
 NwP.nLogMsg    = 0;
 NwP.file       = NULL;
 NwP.startTime  = 0;
 NwP.lifeTime   = 200000;
 FrStrCpy(&(NwP.dataDir),"/export/scratch/ndas");
 FrStrCpy(&(NwP.tag), "H0:PEM-LVEA2_V1 H0:PEM-LVEA_SEIS*");
 FrStrCpy(&(NwP.htmlName),"/opt/CDS/a/apacheFiles/htdocs/gds/"
                          "monitor_reports/NdasWriter/NdasWriter_status.html");

           /*------------------- Loop over the parameters -------------------*/

 for (i=1; i<argc; i=i+2)
   {if      (strcmp(argv[i],"-d") == 0) {FrStrCpy(&(NwP.dataDir), (char*)argv[i+1]);}
    else if (strcmp(argv[i],"-t") == 0) {FrStrCpy(&(NwP.tag),     (char*)argv[i+1]);}
    else if (strcmp(argv[i],"-s") == 0) {FrStrCpy(&(NwP.lscName), (char*)argv[i+1]);}
    else if (strcmp(argv[i],"-h") == 0) {FrStrCpy(&(NwP.htmlName),(char*)argv[i+1]);}
    else if (strcmp(argv[i],"-n") == 0) {NwP.framePerFile = atoi(argv[i+1]);}
    else if (strcmp(argv[i],"-l") == 0) {NwP.lifeTime     = atoi(argv[i+1]);}
    else if (strcmp(argv[i],"-v") == 0) {NwP.verbose      = FR_YES; i--;}
    else if (strcmp(argv[i],"-f") == 0) {NwP.freqLSC      = atoi(argv[i+1]);}
    else if (strcmp(argv[i],"-p") == 0) {NwP.htmlPeriod   = atoi(argv[i+1]);}
    else
       {printf("error while reading parameters\n");}}

           /*------------------ Check start/stop parameters------------------*/

 sprintf(NwP.config,
     "  NdasWriter version   : %s\n"
     "  # of frame per file  : %d\n"
     "  LSC channelfrequency : %dkHz\n"
     "  Output tag           : %s\n"
     "  Data   directory     : %s\n"
     "  HTML report file     : %s\n"
     "  HTML report period   : %d\n",
	 NwVersion,   NwP.framePerFile, NwP.freqLSC, NwP.tag,
         NwP.dataDir, NwP.htmlName,     NwP.htmlPeriod);

 printf("-----------Parameters used--------------\n%s",NwP.config);
 printf("----------------------------------------\n");
 fflush(stdout);
 
 return;
}
/**--------------------------------------------------------------------------*/
void DMT_Halt(void) {
/**--------------------------------------------------------------------------*/
    printf("Monitor has completed\n");
     
    FrFileOEnd(NwP.file);
}

/*---------------------------------------------------------------------------*/
void NwCleanOldFiles(char *dirName, int gpsMin)
/*---------------------------------------------------------------------------*/
/* This function removes all files with a GPS time older than gpsMin         */
/*---------------------------------------------------------------------------*/
{DIR  *dirp;
 struct dirent *entry;
 char fileName[512],exp,extension;
 int  gps,len;

 dirp  = opendir(dirName);

 while((entry = readdir(dirp)) != NULL) {

    if(entry->d_name[1]  != '-') continue;
    if(entry->d_name[11] != '-') continue;

    sscanf(entry->d_name,"%c-%d-%d.%c",&exp,&gps,&len,&extension);
    if(extension != 'F') continue;
    if(gps > gpsMin) continue;

    sprintf(fileName,"%s/%s",dirName, entry->d_name);
    if(NwP.verbose == FR_YES) printf("remove file:%s\n",fileName);
    remove(fileName);}

 closedir(dirp);

 return;}

/*---------------------------------------------------------------------------*/
void NwLog(struct FrameH* frame, char* format, ...) {
/*---------------------------------------------------------------------------*/
 int i;
 va_list    args;
 char       text[1024], mytime[50];
 time_t tp;

 va_start (args, format);
 vsprintf (text, format, args);
 va_end (args);

 i = NwP.nLogMsg%NWMAXMSG;

 if(NwP.logMsg[i] == NULL) NwP.logMsg[i] = malloc(1024);
 if(NwP.logMsg[i] == NULL) return;
 
 tp = time(NULL);
 strcpy(mytime, ctime(&tp));
 mytime[20] = '\0';
 sprintf(NwP.logMsg[i]," (%d %s) %s",frame->GTimeS, &(mytime[11]), text);

 if(NwP.verbose == FR_YES) printf("\n >>> %s\n",NwP.logMsg[i]);

 NwP.nLogMsg++;

 return;}

/*---------------------------------------------------------------- NwStatus--*/
void NwStatus(FrameH *frame, double mean)
/*---------------------------------------------------------------------------*/
{FILE *fp;
 time_t tp;
 int index,i;

 if(frame->GTimeS % NwP.htmlPeriod != 0) return;

 fp = fopen(NwP.htmlName,"w");
 if(fp == NULL)
   {printf(" NdasWriter_status.html file open error\n"); 
   return;}

 tp = time(NULL); 
 fprintf(fp,"<html>\n<head>\n"
       "<title>NDAS Writer Status</title>\n</head>\n<body>\n"
       "<center><b><font color=\"#1822CD\"><font size=+3>NdasWriter Status"
       "</font></font></b></center>\n"
       "<hr WIDTH=\"100%%\">\n  <b>\n"
       "<br>Report time: <font color=\"#1822CD\">%s</font> "
       "(update every %d s.)\n"
       "<br>Last frame: <font color=\"#1822CD\">%d</font>",
        ctime(&tp), NwP.htmlPeriod, frame->GTimeS);

 fprintf(fp,
       "<br>Frame processed: <font color=\"#1822CD\">%d</font>\n"
       "<br>Current output file: %s (<font color=\"#1822CD\">%d</font> frames,"
       " <font color=\"#1822CD\">%.3f</font> MBytes)\n"
       "<br>Number of fast/slow channels: %d/%d"
       "<br>Resample LSC-AS_Q channel at %d kHz "
       "<br>Frames per file:%d\n"
       "   File lifetime:%d s\n"
       "<br>Channels saved:%s\n"
       "<hr WIDTH=\"100%%\">\n"
       "<b><font color=\"#1822CD\"><font size=+1>Recent Log messages " 
       "(out of %d log messages):</font></font></b>\n"
       "<ul>\n<font size=-1>",
         NwP.nFrame, NwP.frFName, 
         NwP.file->nFrames,  NwP.file->nBytes/1048576.,
         NwP.nAdcF, NwP.nAdcS, NwP.freqLSC,
         NwP.framePerFile,NwP.lifeTime,NwP.tag,NwP.nLogMsg);

  /*------- Printf Log Messages -----*/

 for(i=NwP.nLogMsg-1; i>=0; i--)
     {if(i<NwP.nLogMsg-NWMAXMSG) break;
      index = i%NWMAXMSG;
      fprintf(fp,"<li><b>%s</b></li>\n",NwP.logMsg[index]);}

 fprintf(fp,"</ul>\n");
 
   /*- end of file ----*/

 fprintf(fp,"</ul>\n<hr WIDTH=\"100%%\"><div align=right>"
	 "For any question, e-mail to mours@ligo.caltech.edu"
         "</div>\n</body>\n</html>");

 fclose(fp);

 return;}
 
/*---------------------------------------------------------------------------*/
void NwFindPulse(FrameH *frame)              /* find Ndas clock pulse at LLO */
/*---------------------------------------------------------------------------*/
{FrVect *vect;
 int i;
 static double tStart, len;
 static FRBOOL on = FR_NO;
 char *input="L1:GDS-GPS_TRIG2";

 vect = FrameGetV(frame,input);
 if(vect == NULL) return;

 for (i=0; i< vect->nData; i++) 
   {if(on == FR_NO) 
        {if(vect->dataS[i] > 10000) 
            {tStart = frame->GTimeS + i*vect->dx[0];
	     on = FR_YES;}}
    else
        {if(vect->dataS[i] <= 10000) 
            {len = frame->GTimeS + i*vect->dx[0] - tStart;
	     on = FR_NO;
             FrEventNew (frame, "NdasClock_L", NULL, input, 
                 tStart, 0., 0., 1000.*len,  100., NULL, NULL, 0);}}}

 return;}

/**--------------------------------------------------------------------------*/
void DMT_Frame(struct FrameH* frame) {
/**--------------------------------------------------------------------------*/
FrAdcData *adc;
static unsigned int lastGPS=0;
static int nDecimate;
char program[256],tmpName[256];
int nFrames, nBytes;

         /*------------------- file close/open ------------------------------*/

 if(frame->GTimeS >= NwP.startTime+NwP.framePerFile)
     {sprintf(tmpName,"%s/../NdasWriter.tmp",NwP.dataDir);
     if(NwP.file != NULL) 
        {nFrames = NwP.file->nFrames;
         nBytes = NwP.file->nBytes;
         FrFileOEnd(NwP.file);
         rename( tmpName,NwP.frFName);
         if(NwP.verbose == FR_YES) 
           printf("Move file:%s\n       to:%s\n", tmpName, NwP.frFName);

        NwCleanOldFiles(NwP.dataDir, frame->GTimeS - NwP.lifeTime);}
     else
        {nFrames = 0;
         nBytes = 0;}

     NwP.startTime = frame->GTimeS - (frame->GTimeS % NwP.framePerFile);
     sprintf(NwP.frFName,"%s/%.1s-%d-%d.F",NwP.dataDir,
              frame->rawData->firstAdc->name,NwP.startTime, NwP.framePerFile);
     NwLog(frame," Open frame file:%s "
            "(previous file: %d frames %.2f MBytes)",
            NwP.frFName, nFrames, nBytes/1048576.);
     sprintf(program,"LIGO NdasWriter %s",NwVersion);
     NwP.file = FrFileONewH(tmpName, 6, program);
     if(NwP.file == NULL)
          {NwLog(frame," Error: frame file open error");
           exit(0);}}

        /*------------------ check if we miss some frame --------------------*/

 NwP.nFrame ++;
 if((lastGPS+1 != frame->GTimeS) && (lastGPS > 0))
   {NwLog(frame,">>> Warning: %d frame(s) missing<<<",
                      frame->GTimeS - lastGPS -1);}
 lastGPS = frame->GTimeS;

 if(NwP.verbose == FR_YES) 
     {printf("GTimeS=%d frame processed:%6d\r",frame->GTimeS, NwP.nFrame);
      fflush(stdout);}

        /*------------------------------------ find the number of ADC-------*/

 NwP.nAdcF = 0;
 NwP.nAdcS = 0;

 if(frame->rawData != NULL) 
   {for(adc = frame->rawData->firstAdc; adc != NULL; adc=adc->next)
     {if(adc->data->nData <= 16) 
           {NwP.nAdcS++;}
      else {NwP.nAdcF++;}}}

        /*--------------------- decimate the common mode signal---------------*/

 if(NwP.freqLSC != 0 && NwP.lscName != NULL)
   {for(adc = frame->rawData->firstAdc; adc != NULL; adc = adc->next)
      {if(strcmp(adc->name+3,NwP.lscName) != 0) continue;
       nDecimate = adc->data->nData/(1024*NwP.freqLSC);
       FrAdcDataDecimate(adc, nDecimate);}}
 

 if(frame->rawData->firstAdc->name[0] == 'L') NwFindPulse(frame);

        /*-------------------- set detector prefix ------*/

 if(frame->detectProc != NULL)
   {if(frame->detectProc->prefix[0] == '*')
      {frame->detectProc->prefix[0] = frame->rawData->firstAdc->name[0];
       frame->detectProc->prefix[1] = frame->rawData->firstAdc->name[1];}}

        /*----------- write data---------------------------*/

 FrameTagAdc(frame, NwP.tag);
 FrameWrite (frame, NwP.file);
 
 NwStatus(frame,0);

 return;}

